//based on the wikipedia search provider
//
const St = imports.gi.St;
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;
const Search = imports.ui.search;
const Atk = imports.gi.Atk;
const SearchDisplay = imports.ui.searchDisplay;
const IconGrid = imports.ui.iconGrid;
const Gio = imports.gi.Gio;
const Soup = imports.gi.Soup;
const Util = imports.misc.util;
const URLHighlighter = imports.ui.messageTray.URLHighlighter;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;
const Prefs = Me.imports.prefs;
const settings = Convenience.getSettings();
let text, button;
let shoppingProvider = "";
const shell_version = imports.misc.config.PACKAGE_VERSION;

const MAX_SEARCH_RESULTS_COLUMNS = 2
const ICON_SIZE = 120;


function starts_with(str1, str2) {
    return str1.slice(0, str2.length) == str2;
}

function is_blank(str) {
    return (!str || /^\s*$/.test(str));
}
const _httpSession = new Soup.SessionAsync();
Soup.Session.prototype.add_feature.call(
    _httpSession,
    new Soup.ProxyResolverDefault()
);
_httpSession.user_agent = 'Gnome-Shell Shopping Search Provider';

function get_icon(url) {
    let result;

    if(url) {
        let textureCache = St.TextureCache.get_default();
        result = textureCache.load_uri_async(url, ICON_SIZE, ICON_SIZE);
    }
    else {
        result = new St.Icon({
            icon_size: ICON_SIZE,
            icon_name: 'shopping',
            style_class: 'shopping-icon-'+settings.get_string(Prefs.SHOP_THEME)
        });

        if(starts_with(shell_version, '3.4')) {
            result.icon_type = St.IconType.FULLCOLOR
        }
    }

    return result;
}
function _hideHello() {
    Main.uiGroup.remove_actor(text);
    text = null;
}

function _showHello() {
    if (!text) {
        text = new St.Label({ style_class: 'helloworld-label', text: "Hello, world! woot" });
        Main.uiGroup.add_actor(text);
    }

    text.opacity = 255;

    let monitor = Main.layoutManager.primaryMonitor;

    text.set_position(Math.floor(monitor.width / 2 - text.width / 2),
                      Math.floor(monitor.height / 2 - text.height / 2));

    Tweener.addTween(text,
                     { opacity: 0,
                       time: 2,
                       transition: 'easeOutQuad',
                       onComplete: _hideHello });
}
const ShoppingResultActor = new Lang.Class({
    Name: 'ShoppingResultActor',

    _init: function(resultMeta) {
        this.actor = new St.Bin({
            style_class: 'shopping-'+settings.get_string(Prefs.SHOP_THEME),
            reactive: true,
            can_focus: true,
            track_hover: true,
            accessible_role: Atk.Role.PUSH_BUTTON
        });

        let content_width = settings.get_int(Prefs.SHOP_RESULT_WIDTH);
        let content_height = settings.get_int(Prefs.SHOP_RESULT_HEIGHT);
        let style_string = 
            'width: '+content_width+'px;'+
            'height: '+content_height+'px;';

        let content = new St.BoxLayout({
            style_class: 'shopping-content-'+settings.get_string(Prefs.SHOP_THEME),
            style: style_string,
            vertical: false
        });
        this.actor.set_child(content);

        if(resultMeta.show_icon) {
            let icon = get_icon(resultMeta.image);
            global.log(resultMeta.title);
            content.add(icon, {
                x_fill: true,
                y_fill: false,
                x_align: St.Align.START,
                y_align: St.Align.MIDDLE
            });
        }

        let details = new St.BoxLayout({
            style_class: 'shopping-details-'+settings.get_string(Prefs.SHOP_THEME),
            vertical: true
        });

        content.add(details, {
            x_fill: true,
            y_fill: false,
            x_align: St.Align.START,
            y_align: St.Align.MIDDLE
        });

        let title = new St.Label({
            text: resultMeta.title,
            style_class: 'shopping-details-title-'+settings.get_string(Prefs.SHOP_THEME),
            style: 'font-size: '+settings.get_int(Prefs.SHOP_TITLE_FONT_SIZE)+'px;'
        });

        details.add(title, {
            x_fill: true,
            y_fill: false,
            x_align: St.Align.START,
            y_align: St.Align.START
        });
        this.actor.label_actor = title;

        let extract_box = new St.BoxLayout({
            vertical: true,
            style_class: 'shopping-details-extract-'+settings.get_string(Prefs.SHOP_THEME),
            style: 'font-size: '+settings.get_int(Prefs.SHOP_EXTRACT_FONT_SIZE)+'px;'
        });

        let extract = new URLHighlighter(
            resultMeta.extract,
            true,
            false
        );

        extract_box.add(extract.actor, {
            x_fill: true,
            y_fill: false,
            x_align: St.Align.END,
            y_align: St.Align.START
        });

        details.add(extract_box, {
            x_fill: false,
            y_fill: true,
            x_align: St.Align.START,
            y_align: St.Align.END
        });
    }
});

const ShoppingProvider = new Lang.Class({
    Name: 'ShoppingProvider',
    Extends: Search.SearchProvider,

    _init: function(title) {
        this.title = title;
        this.async = true;
        this.delay_query = '';
        this.delay_query_id = 0;
    },
  _get_products: function(term, fun) {
        global.log("getting product");
        if(term) {
            term = encodeURIComponent(term);
            let url= "http://products.libertus.co.uk/products/search.php?search="+term+"&type=All&store=co.uk"
            global.log(url);
            let here = this;
            let request = Soup.Message.new('GET', url);

            _httpSession.queue_message(request, function(_httpSession, message) {
                if(message.status_code === 200) {
                    let result = JSON.parse(request.response_body.data);
                    
                    if(result['Item']) {
                        fun.call(here, result);
                    }
                    else {
                        fun.call(here, false);
                    }
                }
                else {
                    fun.call(here, false);
                }
            });
        }
        else {
            fun.call(here, false);
        }
    },

    _search: function(term) {
        global.log("searching for "+term);
        this._get_products(term, function(products) {
            if(products) {
                global.log("found some product");
                let result = [];
                let Items=products['Item'];
                if(Items) {
                    for(let item in Items) {
                        result.push({
                            "id": Items[item]['DetailPageURL'],
                            "title": Items[item]['ItemAttributes']['Title'],
                            "image": Items[item]['SmallImage'] ? Items[item]['SmallImage']['URL']:'',
                            "extract": Items[item]['EditorialReviews'] ? Items[item]['EditorialReviews']['EditorialReview']['Content']:"no description",
                            "show_icon": Items[item]['SmallImage'] ? true:false
                        });
                    }
                }

                if(result.length > 0) {
                    this.searchSystem.pushResults(this, result);
                }
                else {
                    let nothing_found = [{
                        "title": "Shopping Search Provider",
                        "extract": "Your search - "+term+" - did not match any items.",
                        "show_icon": true
                    }]
                    this.searchSystem.pushResults(this, nothing_found);
                };
            }
            else {
                let nothing_found = [{
                    "title": "Shopping Search Provider",
                    "extract": "Your search - "+term+" - did not match any items.",
                    "show_icon": true
                }]
                this.searchSystem.pushResults(this, nothing_found);
            }
        });
    },
        
    
    _parse_query: function(terms_string) {
        let SHOPPING_QUERY_REGEXP = new RegExp(
            "("+settings.get_string(Prefs.SHOP_KEYWORD)+
            "|"+settings.get_string(Prefs.SHOP_KEYWORD)+
            "-(.*?)) (.*)"
        );
        let result = {};
        result.lang = '';
        result.shopping_query = false;

        if(SHOPPING_QUERY_REGEXP.test(terms_string)) {
            result.shopping_query = true;
            let matches = SHOPPING_QUERY_REGEXP.exec(terms_string);
            let language = matches[2];
            let term = matches[3];

            if(!is_blank(language)) {
                result.lang = language.trim();
            }
            if(!is_blank(term)) {
                result.term = term.trim();
            }
        }

        return result;
    },
    
    createResultActor: function(resultMeta, terms) {
        let result = new ShoppingResultActor(resultMeta);
        return result.actor;
    },
    getInitialResultSetAsync: function(terms) {
        if(this.delay_query_id > 0) {
            Mainloop.source_remove(this.delay_query_id);
            this.delay_query_id = 0;
        }

        let terms_string = terms.join(" ");
        let query = this._parse_query(terms_string);

        if(query.shopping_query) {
            let searching = [{
                "title": "Shopping Search Provider",
                "extract": "Searching for '"+query.term+"'...",
                "show_icon": true
            }]
            this.searchSystem.pushResults(this, searching);

            if(query.term) {
                if(query.lang.length > 0) {
                    let shopping_language = query.lang;
                }
                else {
                    let shopping_language = settings.get_string(Prefs.SHOP_DEFAULT_LANGUAGE);
                }

                this.delay_query = query.term;
                this.delay_query_id = Mainloop.timeout_add(
                    settings.get_int(Prefs.SHOP_DELAY_TIME),
                    Lang.bind(this, function() {
                        this._search(this.delay_query);
                    })
                );
            }
            else {
                let welcome = [{
                    "title": "Shopping Search Provider",
                    "extract": "What do you want to buy?",
                    "show_icon": true
                }]
                this.searchSystem.pushResults(this, welcome);
            }
        }
        else {
            this.searchSystem.pushResults(this, []);
        }
    },
    
    getInitialResultSet: function (terms) {
        this.getInitialResultSetAsync(terms);
    },

    getSubsearchResultSetAsync: function(prevResults, terms) {
        this.getInitialResultSetAsync(terms);
    },

    getSubsearchResultSet: function(prevResults, terms) {
        this.getInitialResultSetAsync(terms);
    },
        getResultMetasAsync: function(result, callback) {
        let metas = [];

        for(let i = 0; i < result.length; i++) {
            metas.push({
                'id' : result[i].id,
                'extract' : result[i].extract,
                'title' : result[i].title,
                'show_icon': result[i].show_icon,
                'image': result[i].image
                });
        }

        callback(metas);
    },

    getResultMetas: function(result, callback) {
        this.getResultMetasAsync(result, callback)
    },

    createResultContainerActor: function() {
        let grid = new IconGrid.IconGrid({
            rowLimit: settings.get_int(Prefs.SHOP_RESULTS_ROWS),
            //columnLimit: MAX_SEARCH_RESULTS_COLUMNS,
            xAlign: St.Align.START
        });
        grid.actor.style_class = 'shopping-grid';

        let width = settings.get_int(Prefs.SHOP_RESULT_WIDTH);
        let height = settings.get_int(Prefs.SHOP_RESULT_HEIGHT); 
        let style_string = 
            '-shell-grid-horizontal-item-size: '+width+'px;'+
            '-shell-grid-vertical-item-size: '+height+'px;'
        grid.actor.style = style_string;

        let actor = new SearchDisplay.GridSearchResults(this, grid);
        return actor;
    },
    
    activateResult: function(resultId) {
        global.log(resultId);
        
        if(resultId) {
            try {
                Gio.app_info_launch_default_for_uri(
                    resultId,
                    global.create_app_launch_context()
                );
            }
            catch (e) {
                Util.spawn(['gvfs-open', resultId])
            }
        }
        else {
            // Main.notify("Bad url.");
        }

        return true;
    }
});

function init() {
    button = new St.Bin({ style_class: 'panel-button',
                          reactive: true,
                          can_focus: true,
                          x_fill: true,
                          y_fill: false,
                          track_hover: true });
    let icon = new St.Icon({ icon_name: 'system-run-symbolic',
                             style_class: 'system-status-icon' });

    button.set_child(icon);
    button.connect('button-press-event', _showHello);
    shoppingProvider = new ShoppingProvider('AMAZON');
    
}

function enable() {
    Main.overview.addSearchProvider(shoppingProvider);
    Main.panel._rightBox.insert_child_at_index(button, 0);
        if(starts_with(shell_version, '3.6')) {
        let search_results = Main.overview._viewSelector._searchResults;
        let provider_meta = search_results._metaForProvider(shoppingProvider);
        provider_meta.resultDisplay._grid.actor.style_class = 'shopping-grid';
        provider_meta.resultDisplay._grid._rowLimit = settings.get_int(Prefs.SHOP_RESULTS_ROWS);

        let width = settings.get_int(Prefs.SHOP_RESULT_WIDTH);
        let height = settings.get_int(Prefs.SHOP_RESULT_HEIGHT); 
        provider_meta.resultDisplay._grid.actor.style = 
            '-shell-grid-horizontal-item-size: '+width+'px;'+
            '-shell-grid-vertical-item-size: '+height+'px;'
    }
}

function disable() {
    Main.panel._rightBox.remove_child(button);
}
